# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

def parent_dirpath(fdpath: str): ...

def fmt_seconds(t1, t2=None): ...

def fmt_bytes(num: int): ...

def fmt_file_size(filepath: str): ...

def timestamp(ts): ...

def path2name(path_str: str) -> str: ...
