# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from typing import Any

def timestamp(): ...
def safepath(filepath: Any): ...
def ensure_save(filepath: Any, n: int = ...): ...
def savings(filepath: Any, string: Any, clobber: bool = ...) -> None: ...
def loads(filepath: Any): ...
def sjson(filepath: Any, data: Any, min: bool = ...) -> None: ...
def save_jasm(filepath: Any, data: Any, min: bool = ...): ...
def sjasm(filepath: Any, data: Any, min: bool = ...): ...
def spak(filepath: Any, data: Any, min: bool = ...): ...
def ljson(filepath: Any): ...
def load_jasm(filepath: Any): ...
def ljasm(filepath: Any): ...
def lpak(filepath: Any): ...
def touch(filepath: Any) -> None: ...
